#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef M_PI
#	define M_PI	3.1415926
#endif /* M_PI */

#ifdef	WIN32
# include <windows.h>

#pragma warning(disable : 4244)     // MIPS
#pragma warning(disable : 4136)     // X86
#pragma warning(disable : 4051)     // ALPHA
#endif	/* WIN32 */

#include <GL/gl.h>
#include <GL/glu.h>

#ifdef	WIN32
#	include <GL/glaux.h>
#else	/* WIN32 */
#	include "glaux.h"
#	define CALLBACK
#endif	/* WIN32 */

#include "toglimg.h"

#if 1
#	define		TEXTURE_FILENAME	"syuri.png"
#else
#	define		TEXTURE_FILENAME	"syuri2.png"
#	define		TEXTURE_FILENAME	"syuriG.png"
#	define		TEXTURE_FILENAME	"syuriP.png"
#endif

void CALLBACK reshape(GLsizei width, GLsizei height)
{
	glViewport(0, 0, width, height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)width / (float)height, 1.0, 2000.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(290.0, 470.0, 610.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
}

void DrawPoly()
{
	glBegin(GL_POLYGON);

	glTexCoord2f(0.0, 0.0);
	glVertex3f(-200.0, -200.0, 200.0);
	glTexCoord2f(1.0, 0.0);
	glVertex3f( 200.0, -200.0, 200.0);
	glTexCoord2f(1.0, 1.0);
	glVertex3f( 200.0,  200.0, 200.0);
	glTexCoord2f(0.0, 1.0);
	glVertex3f(-200.0,  200.0, 200.0);

	glEnd();
}

void DrawObject()
{
	glColor3f(1.0, 1.0, 1.0);

	glPushMatrix();
	DrawPoly();
	glPopMatrix();

	glPushMatrix();
	glRotatef(90.0, 0.0, 1.0, 0.0);
	DrawPoly();
	glPopMatrix();

	glPushMatrix();
	glRotatef(180.0, 0.0, 1.0, 0.0);
	DrawPoly();
	glPopMatrix();

	glPushMatrix();
	glRotatef(270.0, 0.0, 1.0, 0.0);
	DrawPoly();
	glPopMatrix();

	glPushMatrix();
	glRotatef(90.0, 1.0, 0.0, 0.0);
	DrawPoly();
	glPopMatrix();

	glPushMatrix();
	glRotatef(270.0, 1.0, 0.0, 0.0);
	DrawPoly();
	glPopMatrix();
}

void CALLBACK display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	DrawObject();

	glFlush();
}

GLboolean readtexture(char *fname)
{
	unsigned char *data;
	TOGLimage	img;


	if ((data = (unsigned char *)calloc(512 * 512 * 4, sizeof(unsigned char))) == NULL) {
		return GL_FALSE;
	}

	toglInitImage(&img);
	if (!toglReadImagePNG(fname, &img)) {
		fprintf(stderr, "Error\n");
		free(data);
		return GL_FALSE;
	}

	gluScaleImage(GL_RGBA, img.m_width, img.m_height, GL_UNSIGNED_BYTE,
		img.m_buffer, 512, 512, GL_UNSIGNED_BYTE, data);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512, 512, 0,
		GL_RGBA, GL_UNSIGNED_BYTE, data);

	free(data);

	toglDeleteImage(&img);

	return GL_TRUE;
}

int main(int argc, char *argv[])
{
	auxInitDisplayMode(AUX_SINGLE | AUX_RGB | AUX_DEPTH);
	auxInitPosition(0, 0, 640, 480);
	auxInitWindow(argv[0]);

	glClearColor(0.45f, 0.5f, 0.96f, 1.0f);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);     
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
	readtexture(TEXTURE_FILENAME);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	auxReshapeFunc(reshape);

	auxMainLoop(display);

	return 0;
}
